#!/bin/sh

#set -v

# Set the current node
NODE=33

KERNEL=2.4.21
USER=`whoami`
ESSID="nistaodv6"
AODV_RUNS=`ps ax|grep aodvd|grep -v grep`
NODE=$1

NodeAddressGlobal=3ffe::$NODE
NodeAddressSite=fec0::$NODE
AodvSiteAddress=fec0:$NODE::$NODE
AodvGlobalAddress=2003:$NODE::$NODE
AodvInterface=aodv0
ManetInterface=eth1

AodvAddress=$AodvGlobalAddress
NodeAddress=$NodeAddressGlobal

# Clean any running aodvd daemon and previously
# assigned addresses to the used interfaces
if [ "$AODV_RUNS" = "" ]; then
 ifconfig $AodvInterface down
 ifconfig $ManetInterface down
 ifconfig $ManetInterface up
else
 killall -9 aodvd
 ifconfig $AodvInterface down
 ifconfig $ManetInterface down
 ifconfig $ManetInterface up
fi

if [ "$USER" = "root" ]; then
 #Remove kmodules
 rmmod ip6_nf_aodv
 rmmod ip6_queue
 
 #Copy new kmodules
 cp -Rf kernel/ip6_nf_aodv.o /lib/modules/$KERNEL/kernel/net/ipv6/.
 cp -Rf kernel/ip6_queue.o /lib/modules/$KERNEL/kernel/net/ipv6/.
 cp -Rf aodvd /sbin/.

#Enable forwarding on ipv4
 echo "1" > /proc/sys/net/ipv4/ip_forward
 echo "1" > /proc/sys/net/ipv4/conf/all/forwarding 

 #Enable forwarding on IPv6
 echo "1" > /proc/sys/net/ipv6/conf/all/forwarding

#cd /lib/modules/$KERNEL/kernel/net/ipv6
#echo "Directory changed to " `pwd`

#Install ipv6, queuing module and AODV module.
 modprobe ipv6
 modprobe ip6_tables
 modprobe ip6_nf_aodv
 modprobe ip6_queue

 #List IPv6 special addresses
 #echo "::1	localhost6 ip6-loopback" >> /etc/hosts
 #echo "fe00::0	ip6-localnet" >> /etc/hosts
 #echo "ff00::0	ip6-localnet" >> /etc/hosts
 #echo "ff02::1	ip6-allnodes" >> /etc/hosts
 #echo "ff02::2	ip6-allrouters" >> /etc/hosts
 #echo "ff02::3	ip6-allhosts" >> /etc/hosts


 #Setup fake network interface aodv0 and add default route.
 ip link set aodv0 up
 #Make aodv0 the default route
 #NOTE: Can not ping6 unknown adresses without it
 ip -6 r a ::/0 dev aodv0
 #Clean up a bit ... it might be an unnecessary step
 ip -6 r d fe80::/64 dev aodv0
 ip -6 r d ff00::/8 dev aodv0

 #Setup Global and Site addresses
 IFACE=`ifconfig $ManetInterface|grep $NodeAddress`
 if [ "$IFACE" = "" ]; then
 ifconfig $ManetInterface inet6 add $NodeAddress/64
 fi

 #Set a global for aodv0 too
 #ifconfig $AodvInterface inet6 add $AodvSiteAddress/64
 ifconfig $AodvInterface inet6 add $AodvAddress/128

# ip -6 r a $NodeAddressGlobal/128 dev $ManetInterface
# ip -6 r a $AodvSiteAddress/0 via $NodeAddressGlobal dev $ManetInterface

 #Set Linux IPv6 routing table garbage collector interval to 1.
 SETINT=`cat /proc/sys/net/ipv6/route/gc_interval`
 
 if [ "$SETINT" = "0" ]; then
 sysctl -w net.ipv6.route.gc_interval=1
 fi

 #Reset ESSID
 /sbin/iwconfig eth1 ESSID $ESSID mode "Ad-Hoc"

 #Now you are ready to start the aodvd daemon.
/sbin/aodvd  -C /etc/aodvd.conf -kl 64 2> /tmp/aodvd-errors.log

fi
