/*
 *	SecAODV IPv6 Project
 *
 *	See NIST-COPYRIGHT and COPYING for license.
 */

#ifndef __AODV6_H__
#define __AODV6_H__ 1

#include <ctype.h>
#include <netinet/ip6.h>
#include "config_ip.h"
#include "util.h"

#define IPV6_ALL_MANET_NODES { { { 0xff,0x02,0,0,0,0,0,0,0,0,0,0,0,0,0,0x1 } } }
#define AODVPORT 654

/*
 * AODV Configurable parameters
 */
#if 0 //Ours
#define ACTIVE_ROUTE_TIMEOUT	 4000 /* 3000  msecs */
#define ALLOWED_HELLO_LOSS	2
#define BLACKLIST_TIMEOUT	RREQ_RETRIES * NET_TRAVERSAL_TIME
#define BROADCAST_RECORD_TIME	2 * NET_TRAVERSAL_TIME
#define K					3  /* 5 */
#define DELETE_PERIOD		(K * max(ACTIVE_ROUTE_TIMEOUT, ALLOWED_HELLO_LOSS * HELLO_INTERVAL))
#define HELLO_INTERVAL		2000 /* 1000 msecs */
#define LOCAL_ADD_TTL		2
#define MAX_REPAIR_TTL		0.3 * NET_DIAMETER
#define MY_ROUTE_TIMEOUT	K * ACTIVE_ROUTE_TIMEOUT
#define NET_DIAMETER		30 /* 35 */
#define NEXT_HOP_WAIT		NODE_TRAVERSAL_TIME + 10
#define NODE_TRAVERSAL_TIME	30 /* 40 */
#define REV_ROUTE_LIFE		NET_TRAVERSAL_TIME
#define LOCAL_ROUTE_LIFE	20000 /* msec = 20sec */
#define NET_TRAVERSAL_TIME	NODE_TRAVERSAL_TIME*NET_DIAMETER /*3 * NODE_TRAVERSAL_TIME * NET_DIAMETER / 2*/
#define PATH_DISCOVERY_TIME	2 * NET_TRAVERSAL_TIME
#define RREQ_RETRIES		2
#define TTL_START		1
#define TTL_INCREMENT		2
#define TTL_THRESHOLD		7
#endif

//RFC 3561 parameters
#define ACTIVE_ROUTE_TIMEOUT	 max( 3000, ALLOWED_HELLO_LOSS * HELLO_INTERVAL)  //3000 /* msecs */
#define ALLOWED_HELLO_LOSS		2
#define BLACKLIST_TIMEOUT		max( RREQ_RETRIES * NET_TRAVERSAL_TIME, (((TTL_THRESHOLD - TTL_START)/TTL_INCREMENT) + 1 + RREQ_RETRIES)
#define K						5 //recommended
#define DELETE_PERIOD			(K * max(ACTIVE_ROUTE_TIMEOUT, HELLO_INTERVAL))
#define HELLO_INTERVAL			1000 /* msecs */
#define LOCAL_ADD_TTL			2
#define MAX_REPAIR_TTL			0.3 * NET_DIAMETER
#define MIN_REPAIR_TTL			//not implemented
#define MY_ROUTE_TIMEOUT		(2 * max(ACTIVE_ROUTE_TIMEOUT, PATH_DISCOVERY_TIME))
#define NET_DIAMETER			35 // 35
#define NET_TRAVERSAL_TIME		2 * (NODE_TRAVERSAL_TIME + NODE_SEC_VERIFY_TIME) * NET_DIAMETER
#define NEXT_HOP_WAIT			NODE_TRAVERSAL_TIME + 10
#define NODE_TRAVERSAL_TIME		40 /* milliseconds */
#ifdef _SECURE_IP_
#define NODE_SEC_VERIFY_TIME	40 /*milliseconds */
#elif defined _SECURE_BOOT_
#define NODE_SEC_VERIFY_TIME	100 /*milliseconds */
#else
#define NODE_SEC_VERIFY_TIME	0
#endif
#define PATH_DISCOVERY_TIME		(NODE_TRAVERSAL_TIME + NODE_SEC_VERIFY_TIME) * NET_DIAMETER //2 * NET_TRAVERSAL_TIME //Too large...
#define RERR_RATELIMIT			10	//not implemented
#define RING_TRAVERSAL_TIME		2 * NODE_TRAVERSAL_TIME * (TTL_VALUE + TIMEOUT_BUFFER)	//not implemented
#define RREQ_RETRIES			2
#define RREQ_RATELIMIT			10
#define TIMEOUT_BUFFER			2
#define RREQ_RETRIES			2
#define TTL_START				1
#define TTL_INCREMENT			2
#define TTL_THRESHOLD			7
#define TTL_VALUE				//not implemented

#define BROADCAST_RECORD_TIME	2 * NET_TRAVERSAL_TIME
#define REV_ROUTE_LIFE		max( ACTIVE_ROUTE_TIMEOUT, (2 * NET_TRAVERSAL_TIME))/* milliseconds */
#define LOCAL_ROUTE_LIFE	30000 /* msec = 30sec */

#define SECURITY_DELAY		   50 /* msec */


extern int helloInterval;
extern int deletePeriod;
extern int activeRouteTimeout;
extern int myRouteTimeout;
extern int reverseRouteLife;


/*
 * Traffic direction.  Controls use of G bit.
 */
#define DIREC_ANY 0
#define DIREC_UNI 1
#define DIREC_BI  2

/*
 * Flags
 */
#define RREQ_F_JOIN		0x80
#define RREQ_F_REPAIR		0x40
#define RREQ_F_GRATUITOUS	0x20
#define RREQ_F_DEST_ONLY	0x10
#define RREQ_F_UNKNOWN_SEQ	0x08

#define RREP_F_REPAIR		0x80
#define RREP_F_ACK		0x40

#define RERR_F_NODELETE		0x80

/* 
 * Route Request (RREQ) Message Format
 *
 *   0                   1                   2                   3
 *   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |     Type      |J|R|G|D|U|   Reserved          |   Hop Count   |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                        32-bit RREQ ID                         |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |              32-bit Destination Sequence Number               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |               32-bit Originator Sequence Number               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                                                               |
 *  |                128-bit Destination IP Address                 |
 *  |                                                               |
 *  |                                                               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                                                               |
 *  |                 128-bit Originator IP Address                 |
 *  |                                                               |
 *  |                                                               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

struct aodv6_rreq {
	u8	type;
	u8	flags;
	u8	reserved;
	u8	hopcount;
	u32	rreq_id;
	u32	dest_seq;
	u32	orig_seq;
	struct in6_addr dest;
	struct in6_addr orig;
#ifdef _SECURE_BOOT_
	u32 signLen;				//real length
#endif
#ifdef _SECURE_IP_
	u32 keyLen;					//real length
	u32 expLen;
	unsigned char pubExp[32];
	unsigned char pubKey[RSA_KEY_LENGTH];
#endif
#ifdef _SECURE_BOOT_
	unsigned char sign[RSA_KEY_LENGTH];
#endif
} __attribute__ ((packed));


/* 
 * Route Reply (RREP) Message Format
 *
 *   0                   1                   2                   3
 *   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |     Type      |R|A|   Reserved  |Prefix Length|   Hop Count   |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |              32-bit Destination Sequence Number               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                                                               |
 *  |                128-bit Destination IP address                 |
 *  |                                                               |
 *  |                                                               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                                                               |
 *  |                 128-bit Originator IP address                 |
 *  |                                                               |
 *  |                                                               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                           Lifetime                            |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 * Notes:
 *
 * - Prefix Size changed to Prefix Length and grown from 5 to 7 bits
 */

struct aodv6_rrep {
	u8	type;
	u8	flags;
	u8	prefix_len;
	u8	hopcount;
	u32	dest_seq;
	struct in6_addr dest;	//Addr of the dest for which route is supplied 
	struct in6_addr orig;	//Addr of source node which requested the RREQ
	u32	lifetime;
#ifdef _SECURE_BOOT_
	u32 signLen;				//real length
#endif
#ifdef _SECURE_IP_
	u32 keyLen;					//real length
	u32 expLen;
	unsigned char pubExp[32];	
	unsigned char pubKey[RSA_KEY_LENGTH];
#endif
#ifdef _SECURE_BOOT_
	unsigned char sign[RSA_KEY_LENGTH];
#endif
} __attribute__ ((packed));

/* 
 * Route Error (RERR) Message Format
 *
 *   0                   1                   2                   3
 *   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |     Type      |N|          Reserved           |   DestCount   |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |         Unreachable Destination Sequence Number (1)           |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-|
 *  |                                                               |
 *  |      Unreachable Destination IPv6 Address (1) (128-bits)      |
 *  |                                                               |
 *  |                                                               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |Additional Unreachable Destination Sequence Numbers (if needed)|
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                                                               |
 *  | Additional Unreachable Destination IPv6 Addresses (if needed) |
 *  |                                                               |
 *  |                                                               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

struct destination {
	u32	seq;
	struct in6_addr addr;
} __attribute__ ((packed));

struct aodv6_rerr {
	u8	type;
	u8	flags;
	u8	reserved;
	u8	destcount;
	struct destination dests[0];
} __attribute__ ((packed));

struct dest {
	struct destination s_dest;
	struct dest *next;
} __attribute__ ((packed));

/*
 * Route Reply Acknowledgement (RREP-ACK) Message Format
 *
 *   0                   1
 *   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |     Type      |   Reserved    |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

struct aodv6_rrep_ack {
	u8	type;
	u8	reserved;
} __attribute__ ((packed));

/*
 * Extensions
 *
 * RREQ and RREP messages may have extensions in the following format.
 *
 *   0                   1                   2                   3
 *   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |     Type      |    Length     |     type-specific data ...
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * 
 */

struct extension {
	u8 type;
	u8 length;
	/* type specific data follows */
	u8 data[0];
} __attribute__ ((packed));

/*
 * Hello Interval Extension Format
 *
 *   0                   1                   2                   3
 *   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |     Type      |    Length     |       Hello Interval ...      |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  | ... Hello Interval, continued |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

struct aodv6_ext_hello_int {
	u8 type;
	u8 lenght;
	u32 interval;
} __attribute__ ((packed));

/*
 * Timestamp Extension Format
 *
 *   0                   1                   2                   3
 *   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *                                  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *                                  |     Type      |    Length     |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                                                               |
 *  +                    Timestamp in NTP Format                    +
 *  |                                                               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

struct aodv6_ext_ntp_timestamp {
	u8 type;
	u8 length;
	u32 timestamp[2];
} __attribute__ ((packed));

/*
 * Gateway Info Extension Format
 *
 *   0                   1                   2                   3
 *   0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |     Type      |    Length     | Prefix Length |A|M|O| Reserved|
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                        Valid Lifetime                         |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                       Prefered Lifetime                       |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                          Preference                           |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |                                                               |
 *  :                 128-bit Gateway IPv6 Address                  :
 *  |                                                               |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 */

struct aodv6_ext_gateway {
	u8	type;
	u8	length;
	u8	prefix_len;
	u8	autonomous: 1,
		managed: 1,
		other: 1,
		reserved: 5;
	u32	valid_lifetime;
	u32	pref_lifetime;
	u32	preference;
	struct in6_addr gw_addr;
} __attribute__ ((packed));

/*
 * Data Flooding hop-by-hop options
 *
 * AODV for IPv6 defines two IPv6 hop-by-hop options to be used in
 * flooding data without a route.
 */

struct hbhopt_flood_data {
	u8	type;
	u8	length;
	u32	join:1,
		repair:1,
		gratuitous:1,
		reserved:21,
		hopcount:8;
	u32	flood_id;
	u32	dest_seq;
	u32	orig_seq;
} __attribute__ ((packed));

struct hbhopt_flood_data_reply {
	u8	type;
	u8	length;
	u32	repair:1,
		ack:1,
		reserved:15,
		prefix_len:7,
		hopcount:8;
	u32	dest_seq;
	u32	lifetime;
} __attribute__ ((packed));

/*
 * AODV Message types
 */
#define MSG_RREQ     1  /* Route Request   */
#define MSG_RREP     2  /* Route Reply     */
#define MSG_RERR     3  /* Route Error     */
#define MSG_RREP_ACK 4  /* Route Reply Ack */

/*
 * Extension types
 */
#define EXT_HELLO_INTERVAL 1
#define EXT_NTP_TIMESTAMP  3
#define EXT_MANET_GATEWAY  4

/*
 * AODV Message related error codes
 */
#define E_MSGLEN 10
#define E_UNKMSG 11


int aodv6_rrep_recv(struct msghdr *msg, size_t len);
int aodv6_rreq_recv(struct msghdr *msg, size_t len);
int aodv6_rerr_recv(struct msghdr *msg, size_t len);
int aodv6_rrep_ack_recv(struct msghdr *msg, size_t len);

int aodv6_rrep_send(struct in6_addr *dst, struct aodv6_rrep *rrep, size_t len);
int aodv6_rreq_send(struct in6_addr *dst, int hoplimit, struct aodv6_rreq *rreq, size_t len);
int aodv6_rerr_send(struct in6_addr *dst, struct aodv6_rerr *rerr, size_t len);
int aodv6_rrep_ack_send(struct in6_addr *dst);

void aodv6_hello_start(int ifindex, int interval, int setopt);
int request_route(unsigned long requester, struct in6_addr *addr, int type);
int no_forward_route(struct in6_addr *dst, struct in6_addr *sender);

/*
void copy_rreq_secData(struct aodv6_rreq *new, struct aodv6_rreq *old);
void copy_rrep_secData(struct aodv6_rrep *new, struct aodv6_rrep *old);
int sign_rreq(struct aodv6_rreq *rreq);
int sign_rrep(struct aodv6_rrep *rrep);

void copy_secData(void *new, void *old, int type);
int sign_msg(void *data, int type);
*/

#endif /* __AODV6_H__ */
