/*
 *	SecAODV IPv6 Project
 *
 *	See NIST-COPYRIGHT and COPYING for license.
 */

#include <sys/stat.h>
#include <unistd.h>

#include "aodv6.h"
#include "config_ip.h"
#include "util.h"
#include "popup.h"

extern int xdbg;
extern int xtr;
int signInfo = 0;
int _hdr = 1;

#define DES3KEY "stopbitchingstartworking"

/* Release information */
const char *description = "Secure Bootstrapping AODV for IPv6";
const char *version = "1.0";
const char *author = "Secure Bootstrapping: Michaela Iorga,\n\
	based on HUT-AODV: Antti J. Tuominen";

#ifdef _SECURE_IP_

unsigned char *p_mod = NULL;
u16 p_mod_len = 0;
u32 p_exp = RSA_F4;
RSA *key = NULL;
unsigned int _RSAkeylen = RSA_KEY_LENGTH;

#ifdef _eAODVD_
static unsigned char index_64[256] = {
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,62, XX,XX,XX,63,
  52,53,54,55, 56,57,58,59, 60,61,XX,XX, XX,EQ,XX,XX,
  XX, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
  15,16,17,18, 19,20,21,22, 23,24,25,XX, XX,XX,XX,XX,
  XX,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
  41,42,43,44, 45,46,47,48, 49,50,51,XX, XX,XX,XX,XX,

  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
  XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX, XX,XX,XX,XX,
};
#endif

void dumpBadIP(struct in6_addr *addr, char *pubKey)
{
	int fh = 0, rc = 0;

	if((fh = open("/tmp/blackList", O_RDWR | O_CREAT | O_TRUNC, 
				S_IREAD | S_IWRITE | S_IRGRP | S_IROTH)) == 0)
		return;
	else
	{
		char entry[256];
		memset(entry,0,256);
		sprintf(entry,"%x:%x:%x:%x:%x:%x:%x:%x %s\n",NIP6ADDR(addr),pubKey);
		rc = write(fh,entry,strlen(entry));
		close(fh);
		fh = 0;
	}
	
	return;
}

int config_secIP(int scope, unsigned char *key, int klen, 
					const char *netID, struct in6_addr *a)
{
	int i=0;
	unsigned char *m;
	unsigned char *p;
	unsigned char digest[MD5_DIGEST_LENGTH]; //16
	unsigned char id[8];
	unsigned char umask = 0x02; //OR it to set universal bit = 1
	unsigned char gmask = 0xfe;	//AND it to set group bit = 0

	if(xtr) fprintf(stdout,"=== %s ===\n",__FUNCTION__);

	if(a == NULL) 
	{
		return -1;
	}

	memset(a, 0, sizeof(struct in6_addr));
	memset(id,0,8);
	memset(digest,0,MD5_DIGEST_LENGTH);

#ifdef _eAODVD_
	if(_eIP)
	{
		/* Set only the GLOBAL Address
		 * Ignore the Site Addr for now
		 */
		if(scope == SCOPE_GLOBAL)
		{
			if(evilAddr == NULL)
				return -1;
	
			if (inet_pton(AF_INET6, evilAddr, a) <= 0)
				return -1;
			else
				if(xdbg) fprintf(stdout,
						"===> New Evil IP : %x:%x:%x:%x:%x:%x:%x:%x\n",
					NIP6ADDR(a));
		}
		
		if(xtr) fprintf(stdout,"+++ %s +++\n",__FUNCTION__);
		return 0;

		
	}
#endif

	/* Get the address for the network ID from string netID */
	if (inet_pton(AF_INET6, netID, a) <= 0)
		return -1;

	/* Run MD5 over the public key (modulus) */
	/*{
	MD5_CTX md5ctx;
	MD5_Init (&md5ctx);
	MD5_Update (&md5ctx, key, klen);
	MD5_Final ( digest, &md5ctx);
	}*/

	//Faster way:
	p = MD5(key,klen,digest);
	if(xdbg) printBin(stdout,p,MD5_DIGEST_LENGTH);
	
	m  = p+8;
	if(xdbg) printBin(stdout,m,MD5_DIGEST_LENGTH/2);

	for(i=0;i<MD5_DIGEST_LENGTH/2; i++,p++,m++)
		id[i] = (*p)^(*m);

	/* Set the g/u bits 
	 * cccc ccug cccc cccc | cccc cccc ... 
	 *        u=universal (1=universal, 0=local)
	 *         g=individual/group
	 */
	if(xdbg) printBin(stdout,id,8);
	if(scope == SCOPE_GLOBAL)
	{
		//fprintf(stdout, "SCOPE_GLOBAL\n");
		id[0] = (id[0] | umask) & gmask;
	}
	else if (scope == SCOPE_SITE)
	{
		//fprintf(stdout, "SCOPE_SITE\n");
		id[0] = id[0] | 0x03; 
	}

	//printBin(stdout,(unsigned char *)a,sizeof(struct in6_addr));
	//printBin(stdout,id,8);
	
	memcpy(&a->s6_addr32[2],(u32 *)&id[0], sizeof(u32));
	memcpy(&a->s6_addr32[3],(u32 *)&id[4], sizeof(u32));

	if (_hdr)
	{
		fprintf(stdout,"\n\tSETTINGS : \n\t----------\n");
		_hdr = 0;
	}

	if(scope == SCOPE_GLOBAL)
	{
		fprintf(stdout,"\tNew Global IP = %x:%x:%x:%x:%x:%x:%x:%x\n",
				NIP6ADDR(a));
	}
	else if (scope == SCOPE_SITE)
	{
		fprintf(stdout,"\tNew Site IP   = %x:%x:%x:%x:%x:%x:%x:%x\n",
				NIP6ADDR(a));
	}
	//printBin(stdout,(unsigned char *)a,16);

	if(xtr) fprintf(stdout,"+++ %s +++\n",__FUNCTION__);
	
	return 0;
}

#ifdef _HACK_TEST_
int verify_secIP(int scope, unsigned char *key, int klen, struct in6_addr *addr)
{
	return 0;
}
int EVP_sign(unsigned char *data, unsigned int dataLen,
           unsigned char *sigret, unsigned int *siglen, RSA *rsakey)
{
	return 0;
}

int EVP_verify(unsigned char *data, unsigned int dataLen,
           unsigned char *sigbuf, unsigned int siglen, RSA *rsakey)
{
	return 0;
}

int sign_msg(void *rtMsg, int type)
{
	return 0;
}

int verify_sign(void * rtMsg, int msg_type)
{
	return 0;	
}

int copy_secData(void *new, void *old, int type)
{
	return 0;
}
#else
int verify_secIP(int scope, unsigned char *key, int klen, struct in6_addr *addr)
{
	int i=0, rc=0;
	unsigned char *m;
	unsigned char *p;
	unsigned char digest[MD5_DIGEST_LENGTH]; //16
	unsigned char id[8];
	unsigned char umask = 0x02; //OR it to set universal bit = 1
	unsigned char gmask = 0xfe;	//AND it to set group bit = 0
	unsigned char ip[16];

	if(xtr) fprintf(stdout,"=== %s ===\n",__FUNCTION__);

	if(addr == NULL) 
	{
		return -1;
	}
	
	memset(id,0,8);
	memset(digest,0,MD5_DIGEST_LENGTH);
	memset(ip,0,16);

	/* Copy address */
	if(memcpy(ip, addr,sizeof(struct in6_addr)) == NULL)
	{
		return -1;
	}
	if(xdbg) printBin(stdout, ip,16);
	
	/* Run MD5 over the public key (modulus) */
	/*{
	MD5_CTX md5ctx;
	MD5_Init (&md5ctx);
	MD5_Update (&md5ctx, key, klen);
	MD5_Final ( devID, &md5ctx);
	}*/

	if(xdbg) 
	{
		fprintf(stdout,"Key of length %d is :\n", klen);
		printBin(stdout,key,klen);
		fprintf(stdout,"Computing MD5 and deriving the IP\n");
	}
	
	//Faster way:
	p = MD5(key,klen,digest);
	if(xdbg) printBin(stdout,p,MD5_DIGEST_LENGTH);
	
	m  = p+8;
	if(xdbg) printBin(stdout,m,MD5_DIGEST_LENGTH/2);

	for(i=0;i<MD5_DIGEST_LENGTH/2; i++,p++,m++)
		id[i] = (*p)^(*m);
	if(xdbg) printBin(stdout,id,8);

	/* Set the g/u bits 
	 * cccc ccug cccc cccc | cccc cccc ... 
	 *        u=universal (1=universal, 0=local)
	 *         g=individual/group
	 */
	if(scope == SCOPE_GLOBAL)
	{
		id[0] = (id[0] | umask) & gmask;
	}
	else if (scope == SCOPE_SITE)
	{
		id[0] = (id[0] | 0x03); 
	}
	if(xdbg) {
		printBin(stdout,id,8);

		fprintf(stdout,"Compare ip and id:\n");
		printBin(stdout, &ip[sizeof(struct in6_addr)/2], 
				sizeof(struct in6_addr)/2);
		printBin(stdout,id, (sizeof(struct in6_addr)/2));
	}
	
	rc = memcmp(&ip[sizeof(struct in6_addr)/2],id,
				(sizeof(struct in6_addr)/2));

	if(rc)
	{
		char msg[64];
		fprintf(stdout, "\n---> SECURITY ALERT:\n\tCOMPROMISED IP "); 
		print_in6addr(stdout, addr);
		sprintf(msg,"COMPROMISED IP %x:%x:%x:%x:%x:%x:%x:%x\n",NIP6ADDR(addr));
		TellUI(sockd,msg);
		dumpBadIP(addr,"<--- bad-IP");
	}
	else if (signInfo) {
		fprintf(stdout, "\n---> SECURITY MSG:\n\tVERIFIED IP "); 
		print_in6addr(stdout, addr);
	}
		
		

	return rc;
}


int copy_secData(void *new, void *old, int type)
{
	
	if(type == MSG_RREQ)
	{
		struct aodv6_rreq *newMsg = (struct aodv6_rreq *)new;
		struct aodv6_rreq *oldMsg = (struct aodv6_rreq *)old;

		newMsg->keyLen = oldMsg->keyLen;
		newMsg->expLen = oldMsg->expLen;

		memset(newMsg->pubExp,0,sizeof(newMsg->pubExp));
		memcpy(newMsg->pubExp,oldMsg->pubExp,sizeof(oldMsg->pubExp));
		memset(newMsg->pubKey,0,RSA_KEY_LENGTH);
		memcpy(newMsg->pubKey,oldMsg->pubKey,oldMsg->keyLen);

#ifdef _SECURE_BOOT_
		newMsg->signLen = oldMsg->signLen;
		memset(newMsg->sign,0,RSA_KEY_LENGTH);
		memcpy(newMsg->sign, oldMsg->sign,oldMsg->signLen);
#endif

	} else if (type == MSG_RREP)
	{
		struct aodv6_rrep *newMsg = (struct aodv6_rrep *)new;
		struct aodv6_rrep *oldMsg = (struct aodv6_rrep *)old;

		newMsg->keyLen = oldMsg->keyLen;
		newMsg->expLen = oldMsg->expLen;
		
		memset(newMsg->pubExp,0,sizeof(newMsg->pubExp));
		memcpy(newMsg->pubExp,oldMsg->pubExp,sizeof(oldMsg->pubExp));
		memset(newMsg->pubKey,0,RSA_KEY_LENGTH);
		memcpy(newMsg->pubKey,oldMsg->pubKey,oldMsg->keyLen);

#ifdef _SECURE_BOOT_
		newMsg->signLen = oldMsg->signLen;
		memset(newMsg->sign,0,RSA_KEY_LENGTH);
		memcpy(newMsg->sign, oldMsg->sign,oldMsg->signLen);
#endif

	} else 
		return -1;

	return 0;
}

#ifdef _SECURE_BOOT_

int EVP_sign(unsigned char *data, unsigned int dataLen,
           unsigned char *sigret, unsigned int *siglen, RSA *rsakey)
{
	int rc = 0;
	EVP_MD_CTX *ctx = NULL;
	EVP_PKEY *key = NULL;

	key = EVP_PKEY_new();
	if(key == NULL)
	{
		if(xdbg) fprintf(stderr,"EVP_PKEY_new() failed.\n");
		goto final;	
	}
	
	rc = EVP_PKEY_set1_RSA(key, rsakey);
	if(rc == 0)
	{
		if(xdbg) fprintf(stderr, "Can't assigne rsa key to evp key.\n");
		goto final;
	}
	
	//ctx = EVP_MD_CTX_create(); //does not exist for older versions/iPaq
	ctx = ( EVP_MD_CTX *)malloc(sizeof( EVP_MD_CTX));
	if(ctx == NULL)
	{
		if(xdbg) fprintf(stderr, "Can't create CTX object.\n");
		goto final;
	}
	memset(ctx,0,sizeof(EVP_MD_CTX));
		
	/*rc = EVP_SignInit_ex(ctx, EVP_md5(), NULL); 
		if(rc == 0)	//Error
	{
		if(xdbg) fprintf(stderr,"EVP_SignInit() failed.\n");
		goto final;
	}
	*/

	EVP_SignInit(ctx, EVP_md5());
	EVP_SignUpdate(ctx, data, dataLen);
/* -0.9.7 returns value but iPaq doesn't
	rc = EVP_SignUpdate(ctx, data, dataLen);
	if(rc == 0) //Error
	{
		if(xdbg) fprintf(stderr,"EVP_SignUpdate() failed.\n");
		goto final;
	}
*/
	rc = EVP_SignFinal(ctx, sigret, siglen, key);
	if (rc == 0) //Error
		if(xdbg) fprintf(stderr,"EVP_SignUpdate() failed.\n");
	
final:
	if(key) EVP_PKEY_free(key);
	if(ctx) free(ctx); //EVP_MD_CTX_destroy(ctx);
	return rc;
}

int EVP_verify(unsigned char *data, unsigned int dataLen,
           unsigned char *sigbuf, unsigned int siglen, RSA *rsakey)
{
	int rc = 0;
	EVP_MD_CTX *ctx = NULL;
	EVP_PKEY *key = NULL;

	key = EVP_PKEY_new();
	if(key == NULL)
	{
		if(xdbg) fprintf(stderr,"EVP_PKEY_new() failed.\n");
		goto final;	
	}
	
	rc = EVP_PKEY_set1_RSA(key, rsakey);
	if(rc == 0)
	{
		if(xdbg) fprintf(stderr, "Can't assigne rsa key to evp key.\n");
		goto final;
	}
	
	//ctx = EVP_MD_CTX_create(); //does not exist in older versions & iPaqs
	ctx = (EVP_MD_CTX *)malloc(sizeof(EVP_MD_CTX));
	if(ctx == NULL)
	{
		if(xdbg) fprintf(stderr, "Can't create CTX object.\n");
		goto final;
	}
	memset(ctx,0,sizeof(EVP_MD_CTX));
	
	/*
	rc = EVP_VerifyInit_ex(ctx, EVP_md5(), NULL); 
	if(rc == 0)	//Error
	{
		if(xdbg) fprintf(stderr,"EVP_VerifyInit() failed.\n");
		goto final;
	}
	*/
	EVP_VerifyInit(ctx,EVP_md5());
	EVP_VerifyUpdate(ctx, data, dataLen);
/* opensl 0.9.7 returns value but iPaq doesn't
	rc = EVP_VerifyUpdate(ctx, data, dataLen);
	if(rc == 0) //Error
	{
		if(xdbg) fprintf(stderr,"EVP_VerifyUpdate() failed.\n");
		goto final;
	}
*/
	rc = EVP_VerifyFinal(ctx, sigbuf, siglen, key);
	if (rc == 0) //Error
		if(xdbg) fprintf(stderr,"EVP_VerifyUpdate() failed.\n");
	
final:
	if(key) EVP_PKEY_free(key);
	if(ctx) free(ctx); //EVP_MD_CTX_destroy(ctx); //doesn't exist
	return rc;
}

int sign_msg(void *rtMsg, int type)
{
		u32 *data = NULL;
		u32 *d;
		int ret=0;
		unsigned int dataLen;
		int x = xdbg;
		xdbg = 0; //revert it befor return

		//Testing msg UI
		//TellUI(sockd, "SIGNING IN PROGRESS");

		if(type == MSG_RREQ)
		{
			struct aodv6_rreq *msg;
			dataLen = 3*sizeof(u32)+2*sizeof(struct in6_addr);
			data = (u32 *)malloc(dataLen);
			d=data;
			memset(data,0,dataLen);
		
			msg = (struct aodv6_rreq *)malloc(sizeof(struct aodv6_rreq));
			memset(msg,0,sizeof(struct aodv6_rreq));
			memcpy(msg,rtMsg,sizeof(struct aodv6_rreq));
			
			/* Key setting */
			memset(msg->pubKey,0,RSA_KEY_LENGTH);
			msg->keyLen = p_mod_len;
			memcpy(msg->pubKey, p_mod,p_mod_len);
			memset(msg->pubExp,0,sizeof(msg->pubExp));
			msg->expLen = BN_bn2bin(key->e,msg->pubExp);
	
#ifdef _eAODVD_
			if(_eKey)
			{
				msg->keyLen = evp_mod_len;
				memset(msg->pubKey,0,RSA_KEY_LENGTH);
				memcpy(msg->pubKey, evp_mod,evp_mod_len);
				memset(msg->pubExp,0,sizeof(msg->pubExp));
				msg->expLen = BN_bn2bin(evilRSA->e,msg->pubExp);
			}
#endif
			
			memset(msg->sign,0,RSA_KEY_LENGTH);
			
			/* Signature */
			if(xdbg) fprintf(stdout, "RREQ: DATA-TO-SIGN:\n");
			memcpy(data, &msg->rreq_id,sizeof(u32));
			data++;
			if(xdbg) fprintf(stdout,
					"\tRREQ ID       :  %d\n",ntohl(msg->rreq_id));
			memcpy(data, &msg->dest_seq,sizeof(u32));
			data++;
			if(xdbg) fprintf(stdout,
					"\tDEST SEQ#     :  %d\n",ntohl(msg->dest_seq));
			memcpy(data, &msg->orig_seq,sizeof(u32));
			data++;
			if(xdbg) fprintf(stdout, 
					"\tORIG SEQ#     :  %d\n",ntohl(msg->orig_seq));
			memcpy(data, &msg->dest, sizeof(struct in6_addr));
			data += sizeof(struct in6_addr)/sizeof(u32);
			if(xdbg) fprintf(stdout, 
					"\tDESTINATION IP:  %x:%x:%x:%x:%x:%x:%x:%x\n",
							NIP6ADDR(&msg->dest));
			memcpy(data, &msg->orig, sizeof(struct in6_addr));
			if(xdbg) fprintf(stdout, 
					"\tORIGINATOR IP :  %x:%x:%x:%x:%x:%x:%x:%x\n",
							NIP6ADDR(&msg->orig));
			data = d;

			if(xdbg) {
				fprintf(stdout, "\tRREQ EXP LEN  : %d\n", msg->expLen);
				fprintf(stdout, "\tRREQ EXPONENT : ");
				printBin(stdout, msg->pubExp, msg->expLen);
				fprintf(stdout, "\tRREQ KEY LEN  : %d\n",msg->keyLen);
				fprintf(stdout, "\tRREQ MODULUS  : \n");
				printBin(stdout, msg->pubKey, msg->keyLen);
			}
			/* RSA_sign returns 1 on success 0 otherwise */
			//ret = RSA_sign(NID_md5,(unsigned char *)data,
			ret = EVP_sign((unsigned char *)data, dataLen,
					msg->sign, (unsigned int *)&msg->signLen,key);
			
			if(ret==1)
			{
				if(xdbg) 
				{
					fprintf(stdout,
							"\tRREQ key len  :  %d\n\tSIGNATURE:\n",
							RSA_size(key));
					printBin(stdout, msg->sign, msg->signLen);
				}
			} else
				fprintf(stderr,"\tERROR: RREQ not signed!\n");
			
			memcpy(rtMsg,msg,sizeof(struct aodv6_rreq));

			free(data);
			free(msg);
	
		} else if(type == MSG_RREP) 
		{
			struct aodv6_rrep *msg;
			dataLen = 2*sizeof(u32)+2*sizeof(struct in6_addr);
			data = (u32 *)malloc(dataLen);
			d=data;
			memset(data,0,dataLen);
			
			msg = (struct aodv6_rrep *)malloc(sizeof(struct aodv6_rrep));
			memset(msg,0,sizeof(struct aodv6_rrep));
			memcpy(msg,rtMsg,sizeof(struct aodv6_rrep));

			memset(msg->sign,0,RSA_KEY_LENGTH);
	
			/* Key setting */
			memset(msg->pubKey,0,RSA_KEY_LENGTH);
			msg->keyLen = p_mod_len;
			memcpy(msg->pubKey, p_mod,p_mod_len);
			//msg->pubExp = p_exp;
			memset(msg->pubExp,0,sizeof(msg->pubExp));
			msg->expLen = BN_bn2bin(key->e,msg->pubExp);
	
#ifdef _eAODVD_
			if(_eKey)
			{
				msg->keyLen = evp_mod_len;
				memset(msg->pubKey,0,RSA_KEY_LENGTH);
				memcpy(msg->pubKey, evp_mod,evp_mod_len);
				memset(msg->pubExp,0,sizeof(msg->pubExp));
				msg->expLen = BN_bn2bin(evilRSA->e,msg->pubExp);
			}
#endif

			/* Signature */
			if(xdbg) fprintf(stdout, "RREP: DATA-TO-SIGN \n" );
			memcpy(data, &msg->dest_seq,sizeof(u32));
			data++;
			if(xdbg) fprintf(stdout, 
					"\tDEST SEQ#     :  %d\n",ntohl(msg->dest_seq));
			memcpy(data, &msg->lifetime,sizeof(u32));
			data++;
			if(xdbg) fprintf(stdout, 
					"\tLIFETIME      :  %u\n",ntohl(msg->lifetime));
			memcpy(data, &msg->dest, sizeof(struct in6_addr));
			data += sizeof(struct in6_addr)/sizeof(u32);
			if(xdbg) fprintf(stdout, 
					"\tDESTINATION IP:  %x:%x:%x:%x:%x:%x:%x:%x\n",
							NIP6ADDR(&msg->dest));
			memcpy(data, &msg->orig, sizeof(struct in6_addr));
			if(xdbg) fprintf(stdout, 
					"\tORIGINATOR IP :  %x:%x:%x:%x:%x:%x:%x:%x\n",
							NIP6ADDR(&msg->orig));
			data = d;

			if(xdbg) 
			{
				fprintf(stdout, "\tRREP EXP LEN  : %d\n", msg->expLen);
				fprintf(stdout, "\tRREP EXPONENT : ");
				printBin(stdout, msg->pubExp, msg->expLen);
				fprintf(stdout, "\tRREP KEY LEN  : %d\n",msg->keyLen);
				fprintf(stdout, "\tRREP MODULUS  : \n");
				printBin(stdout, msg->pubKey, msg->keyLen);
				fprintf(stdout, "\tRSA_size(key) : %d\n",RSA_size(key));
				fprintf(stdout, "\tmsg->sign size: %d\n",sizeof(msg->sign));
			}

			/* RSA_sign returns 1 on success 0 otherwise */
			//ret = RSA_sign(NID_md5,(unsigned char *)data,
			ret = EVP_sign((unsigned char *)data,
				dataLen, msg->sign, (unsigned int *)&msg->signLen,key);

			/* Bug fix - keyLen is 0 here*/
			msg->keyLen = RSA_size(key);

			if(ret==1)
			{
				if(xdbg) 
				{
					fprintf(stdout,
							"\tRREP sign len  :  %d/%d\n\tSIGNATURE:\n",
							msg->signLen, RSA_size(key));
					printBin(stdout, msg->sign, msg->signLen);
				}
			} else 
			{
				char buf[120];
				char *pbuf = buf;
				memset(buf,0,sizeof(buf));
				ERR_load_crypto_strings();
				pbuf = ERR_error_string(ERR_get_error(), buf);
				ERR_print_errors_fp(stdout);
				fprintf(stderr,"\tERROR: EVP_sign() returned error \"%s\". \
						\n\tRREP not signed!\n", buf);
			}

			/* TEST 1 */
			/*
			{
				RSA *rsa = RSA_new();
				rsa->e = BN_bin2bn(msg->pubExp, msg->expLen, NULL);
				//rsa->e = key->e;
				rsa->n = BN_bin2bn(msg->pubKey, msg->keyLen, NULL);
				printf("=== rsa === \n");
				RSA_print_fp(stdout, rsa, 0);
				printf("=== key ===\n");
				RSA_print_fp(stdout, key, 0);
				if( 1 == RSA_verify(NID_md5,(unsigned char *)data, dataLen,
					msg->sign, msg->signLen, rsa))
					printf("\n\tSIGNATURE VERIFIED LOCO\n");
				else
					printf("\n\tSIGNATURE NOT VERIFIED LOCO\n");
			} */
			
			memcpy(rtMsg,msg,sizeof(struct aodv6_rrep));

			/* TEST 2
			 * verify_sign returns 0 on success or -1 otherwise 
			 *
			if(!verify_sign(rtMsg, type))
				printf("\n\t GOOD SIGNATURE HERE!!\n");
			else
				printf("\n\t BAD SIGNATURE HERE\n");
			**/
				
			free(data);
			free(msg);

		} else
			ret = -99;

		xdbg = x;
		return ret;
}

int verify_sign(void * rtMsg, int msg_type)
{
	/* Verify signature */
	u32 *data = NULL;
	u32 *d = NULL;
	int ret=0;
	unsigned char *msgSign = NULL;
	unsigned int dataLen; 
	RSA *rsa = RSA_new();
	struct in6_addr *addr;
	int x = xdbg;
	xdbg = 0; //revert it befor return

	/* It core dumps if setting RSA_METHOD_FLAG_NO_CHECK flag !!! */
	/* correct method should be:  RSA_FLAG_SIGN_VER
	if(!(m->flags & RSA_METHOD_FLAG_NO_CHECK))
	{
		printf("\n\tSet flags : RSA_FLAG_SIGN_VER | RSA_METHOD_FLAG_NO_CHECK\n");
		m->flags = RSA_METHOD_FLAG_NO_CHECK;
		ret = RSA_set_method(key, m);
	}
	*/
	
	if(msg_type == MSG_RREQ)
	{
		struct aodv6_rreq *msg;
		msg = (struct aodv6_rreq *)rtMsg;
		dataLen = 3*sizeof(u32)+2*sizeof(struct in6_addr);

		addr = &msg->orig;
		data = (u32 *)malloc(dataLen);
		msgSign = (unsigned char *)malloc(msg->signLen);

		memset(data,0,dataLen);
		memset(msgSign,0,msg->signLen);
		memcpy(msgSign,msg->sign,msg->signLen);
	
		rsa->e = BN_bin2bn(msg->pubExp, msg->expLen , NULL);
		rsa->n = BN_bin2bn(msg->pubKey, msg->keyLen, NULL);

		d = data;
			
		/* Signature */
		if(xdbg) fprintf(stdout, "RREQ: DATA-TO-VERIFY:\n");
		memcpy(data, &msg->rreq_id,sizeof(u32));
		data++;
		if(xdbg) fprintf(stdout, "\tRREQ ID       :  %d\n",ntohl(msg->rreq_id));
		memcpy(data, &msg->dest_seq,sizeof(u32));
		data++;
		if(xdbg) fprintf(stdout, "\tDEST SEQ#     :  %d\n",ntohl(msg->dest_seq));
		memcpy(data, &msg->orig_seq,sizeof(u32));
		data++;
		if(xdbg) fprintf(stdout, "\tORIG SEQ#     :  %d\n",ntohl(msg->orig_seq));
		memcpy(data, &msg->dest, sizeof(struct in6_addr));
		data += sizeof(struct in6_addr)/sizeof(u32);
		if(xdbg) fprintf(stdout, "\tDESTINATION IP:  %x:%x:%x:%x:%x:%x:%x:%x\n",
							NIP6ADDR(&msg->dest));
		memcpy(data, &msg->orig, sizeof(struct in6_addr));
		if(xdbg) fprintf(stdout, "\tORIGINATOR IP :  %x:%x:%x:%x:%x:%x:%x:%x\n",
						NIP6ADDR(&msg->orig));
		data = d;

		if(xdbg)
		{
			fprintf(stdout, "\tRREQ EXP LEN  : %d\n", msg->expLen);
			fprintf(stdout, "\tRREQ EXPONENT : ");
			printBin(stdout, msg->pubExp, msg->expLen);
			fprintf(stdout, "\tRREQ KEY LEN  : %d\n",msg->keyLen);
			fprintf(stdout, "\tRREQ MODULUS  : \n");
			printBin(stdout, msg->pubKey, msg->keyLen);

			fprintf(stdout,"\tRREQ sign len  :  %d/%d\n\tSIGNATURE:\n",
							msg->signLen, RSA_size(rsa));
			printBin(stdout, msg->sign, msg->signLen);
		}

		if(xdbg) printf("Calling EVP_verify()\n");
		
		//ret = RSA_verify(NID_md5,(unsigned char *)data, dataLen,
		ret = EVP_verify((unsigned char *)data, dataLen,
				msgSign, msg->signLen, rsa);

		if(ret != 1)
		{
			int i=0;
			char tell[64];
			unsigned char pubBuf[256];
			unsigned char *encBuf;

			if (xdbg) printf("Calling PopupUI applet with warning!\n");

			memset(tell,0,sizeof(msg));
			memset(pubBuf,0,sizeof(pubBuf));

			pubBuf[i++] = 0x45;
			pubBuf[i++] = msg->expLen;
			memcpy(&pubBuf[i],msg->pubExp,msg->expLen);
			i += msg->expLen;
			pubBuf[i++] = 0x4D;
			pubBuf[i++] = msg->keyLen; //RSA_KEY_LENGTH;
			memcpy(&pubBuf[i],msg->pubKey, msg->keyLen);
			i += msg->keyLen;

			if(i+1 > sizeof(pubBuf))
			{
				fprintf(stderr, "*** ERROR: Wrong data length! Overflow?\n");
				exit(1);
			}
		
			/* Encode pub key */
			encBuf = base64_encode(pubBuf,i+1);
			dumpBadIP(addr,encBuf);

			free(encBuf);

			sprintf(tell,"BOGUS MSG FROM IP %x:%x:%x:%x:%x:%x:%x:%x\n",NIP6ADDR(addr));
			TellUI(sockd,tell);
			
			fprintf(stdout,"---> SECURITY ALLERT: - RREQ from IP ");
			print_in6addr(stdout,addr);
			fprintf(stdout,"\tSIGNATURE VERIFICATION FAILED!\n");
			
			return -1;
		}

		if (signInfo) {
			fprintf(stdout,"\tSIGNATURE VERIFICATION PASSED - RREQ from IP ");
			print_in6addr(stdout,addr);
		}


	} else if (msg_type == MSG_RREP)
	{
		struct aodv6_rrep * msg;
		char type[12];

		memset(type,0,12);

		msg = (struct aodv6_rrep *)rtMsg;
		dataLen = 2*sizeof(u32)+2*sizeof(struct in6_addr);

		data = (u32 *)malloc(dataLen);
		msgSign = (unsigned char *)malloc(msg->signLen);

		addr = &msg->dest;

		memset(data,0,dataLen);
		memset(msgSign,0,msg->signLen);
		memcpy(msgSign,msg->sign,msg->signLen);
	
		rsa->e = BN_bin2bn(msg->pubExp, msg->expLen, NULL);
		rsa->n = BN_bin2bn(msg->pubKey, msg->keyLen, NULL);

		d = data;

		/* Signature */
		if(xdbg) fprintf(stdout, "RREP: DATA-TO-VERIFY \n" );
		memcpy(data, &msg->dest_seq,sizeof(u32));
		data++;
		if(xdbg) fprintf(stdout, "\tDEST SEQ#     :  %d\n",ntohl(msg->dest_seq));
		memcpy(data, &msg->lifetime,sizeof(u32));
		data++;
		if(xdbg) fprintf(stdout, "\tLIFETIME      :  %u\n",ntohl(msg->lifetime));
		memcpy(data, &msg->dest, sizeof(struct in6_addr));
		data += sizeof(struct in6_addr)/sizeof(u32);
		if(xdbg) fprintf(stdout, "\tDESTINATION IP:  %x:%x:%x:%x:%x:%x:%x:%x\n",
							NIP6ADDR(&msg->dest));
		memcpy(data, &msg->orig, sizeof(struct in6_addr));
		if(xdbg) fprintf(stdout, "\tORIGINATOR IP :  %x:%x:%x:%x:%x:%x:%x:%x\n",
							NIP6ADDR(&msg->orig));
		data = d;

		if(xdbg) 
		{
			fprintf(stdout, "\tRREP EXP LEN  : %d\n", msg->expLen);
			fprintf(stdout, "\tRREP EXPONENT : ");
			printBin(stdout, msg->pubExp, msg->expLen);
			fprintf(stdout, "\tRREP KEY LEN  : %d\n",msg->keyLen);
			fprintf(stdout, "\tRREP MODULUS  : \n");
			printBin(stdout, msg->pubKey, msg->keyLen);
			fprintf(stdout,"\tRREQ sign len  :  %d/%d\n\tSIGNATURE:\n",
							msg->signLen, RSA_size(rsa));
			printBin(stdout, msg->sign, msg->signLen);
		}
		
		//ret = RSA_verify(NID_md5,(unsigned char *)data, dataLen,
		ret = EVP_verify((unsigned char *)data, dataLen,
				msgSign, msg->signLen, rsa);

		//Determine if this RREP is a Hello msg
		if(ipv6_addr_cmp(&msg->dest,&msg->orig) == 0)
			memcpy(type,"HELLOMSG",9);
		else
			memcpy(type, "RREP",5);

		if(ret != 1)
		{
			int i=0;
			char tell[64];
			unsigned char pubBuf[256];
			unsigned char *encBuf;

			memset(type,0,12);
			memset(tell,0,sizeof(msg));
			memset(pubBuf,0,sizeof(pubBuf));

			pubBuf[i++] = 0x45;
			pubBuf[i++] = msg->expLen;
			memcpy(&pubBuf[i],msg->pubExp,msg->expLen);
			i += msg->expLen;
			pubBuf[i++] = 0x4D;
			pubBuf[i++] = msg->keyLen; //RSA_KEY_LENGTH;
			memcpy(&pubBuf[i],msg->pubKey, msg->keyLen);
			i += msg->keyLen;

			if(i+1 > sizeof(pubBuf))
			{
				fprintf(stderr, "*** ERROR: Wrong data length! Overflow?\n");
				exit(1);
			}
		
			/* Encode pub key */
			encBuf = base64_encode(pubBuf,i+1);
			dumpBadIP(addr,encBuf);

			free(encBuf);

			sprintf(tell,"BOGUS MSG FROM IP %x:%x:%x:%x:%x:%x:%x:%x\n",NIP6ADDR(addr));
			TellUI(sockd,tell);
		
			fprintf(stdout,"---> SECURITY ALLERT: - %s from IP ", type);
			print_in6addr(stdout,addr);
			fprintf(stdout,"\tSIGNATURE VERIFICATION FAILED!\n");
			
			return -1;
		}

		if (signInfo) {
			fprintf(stdout,"\tSIGNATURE VERIFICATION PASSED - %s from IP ",type);
			print_in6addr(stdout,addr);
		}


	} else
		return -1;
		
	free(msgSign);

	xdbg = x;

	/* RSA_verify & EVP_verify return 1 on success and 0 otherwise */
	return 0;
}
#endif //_SECURE_BOOT_

#endif // _HACK
unsigned int BN2bin(const BIGNUM *bn, unsigned char *pmod, int size)
{
	unsigned int len = 0;
	unsigned char *p = NULL;

	if(p!=NULL)
		free(p);

	//BN_num_bytes() returns RSA_KEY_LENGTH/2 instead of RSA_KEY_LENGTH
	len = BN_num_bytes(bn);

	p = (unsigned char *)malloc(RSA_size(key));
	len = BN_bn2bin(bn, p);
	
	memset(pmod,0,size);
		
	if(size < len)
	{
		fprintf(stderr, "Got a keylen %d[bytes] > %d[buffer size]\n",
					len, size);
		return len;
	}
	//fprintf(stdout, "Got keylen=%d[bytes]/size=%d\n",len,size);	
	//printBin(stdout,p,size);

	memcpy(pmod,p,len);
	
	if(xdbg) 
	{
		fprintf(stdout,"Public Key is:\n\texponent : %d\n\tmodulus :\n", p_exp);
		printBin(stdout,pmod,len);
	}
	
	return len;
}

unsigned char * base64_encode(unsigned char *buf, unsigned int len)
{
	unsigned char *ret;
	unsigned int b64_len;

	/* The b64data to data ratio is 3 to 4.
	 * integer divide by3 then multply by 4, add one for NULL terminator.
	 */
	b64_len = (((len + 2) / 3)*4) + 1;
	ret = (unsigned char *)malloc(b64_len);
	EVP_EncodeBlock(ret, buf, len);
	ret[b64_len-1] = 0;
	return ret;
}

#ifdef _eAODVD_	
/* EVP_DecodeBlock not working well... it returns -1
 * Don't trust it

int base64_decode( unsigned char *in, unsigned char *out)
{
	unsigned int b64_len=0,len=0;
	int rc = 0;
	unsigned char b64[256];
	unsigned char dec[256];

	if(out == NULL)
		exit(1);
	
	// The ratio of b64data (=in) to data(=out) is 3 to 4.
	// integer divide by 3 then multply by 4, add one for NULL terminator.	 
	b64_len = strlen(in)+1; 

	//b64_len = (((len + 2) / 3)*4) + 1 (with NULL terminator)
	//=> len = ...
	len = ((b64_len-1)*3)/4;

	printf("\nb64_len=%d len=%d\n",b64_len, len);
		
	printf("%s\n",in);
	memset(b64,0,256);
	memcpy(b64,in,strlen(in)+1);
	memset(dec,0,256);

	// I think it should return the length, instead it returns -1
	// even though the returned data is corect...
	rc = EVP_DecodeBlock(dec, b64, 256);

	//printf("Got rc = %d\n",rc);
	//printBin(stdout,dec,len);
	memcpy(out,dec,len);

	return len;
}
*/
int base64_decode( unsigned char *in, unsigned char *out)
{
    
  int len  = strlen ( in );
  //int rlen = len * 3 / 4;
  unsigned char * end = in + len;
  unsigned char c[4];
  unsigned char * r = out;

	if(out == NULL)
	{
		fprintf(stderr,"ERROR: No out buffer\n");
		exit (1);
	}

  while (in < end) {
    int i = 0;
    do {
      unsigned char uc = index_64[*in++];
      if (uc != INVALID) c[i++] = uc;

      if (in == end) {
	if (i < 4) {
	  if (i)
	    fprintf(stderr,"Premature end of base64 data\n");
	  if (i < 2) goto done;
	  if (i == 2) c[2] = EQ;
	  c[3] = EQ;
	}
	break;
      }
    } while (i < 4);

    if (c[0] == EQ || c[1] == EQ) {
            fprintf(stderr,"Premature padding of base64 data\n"); break; }

     *r++ = (c[0] << 2) | ((c[1] & 0x30) >> 4);

     if (c[2] == EQ) break;
     *r++ = ((c[1] & 0x0F) << 4) | ((c[2] & 0x3C) >> 2);

     if (c[3] == EQ) break;
     *r++ = ((c[2] & 0x03) << 6) | c[3];
  }

 done:
  *r = 0;
  return r - out;
}
#endif

/*
 * DES3_encrypt used to encrypt the prvKey for other applications
 * of the project to use
 * @len = IN/OUT inputData length/encryptedData length
 */
unsigned char * DES3_encrypt(unsigned char *input, int *len)
{
	EVP_CIPHER_CTX ctx;
	unsigned char *res;
	unsigned int tlen = 0;
	char iv[EVP_MAX_BLOCK_LENGTH] = {0,};
	int b = 64;
	int l = *len;
	
	res = (unsigned char *)malloc(*len + EVP_MAX_BLOCK_LENGTH);

	//printf("DATA-TO-ENCRYPT :\n");
	//printBin(stdout,input, *len);
	
	EVP_EncryptInit(&ctx, EVP_des_ede3_cbc(), DES3KEY, iv);
	*len = 0;
	while(l>b)
	{
		EVP_EncryptUpdate(&ctx, &res[*len], &tlen, input, b);
		*len += tlen;
		input += b; 
		l -= b;
	}
	/* last block */
	EVP_EncryptUpdate(&ctx, &res[*len], &tlen, input, l);
	*len += tlen;
	EVP_EncryptFinal(&ctx, &res[*len], &tlen);
	*len += tlen;
	
	//printf("DATA-ENCRYPTED :\n");
	//printBin(stdout,res, *len);
	
	return res;
}

void printBin(FILE *f, unsigned char *k, int len)
{
	int i;
	
	if(k == NULL)
		return;

	fprintf(f,"{ ");
	if(len == 0)
		fprintf(f,"}\n");
	else
	{
		for(i=0;i<len;i++)
		{
			if(i<len-1)
			{
				fprintf(f,"0x%02x,",*k);
				k++;
				if((i+1)%16 == 0)
					fprintf(f, "\n  ");
			}
			else
				fprintf(f,"0x%02x }\n",*k);
		}
	}

	return;
}

#endif //_SECURE_IP_
