/*
 *	SecAODV IPv6 Project
 *
 *	See NIST-COPYRIGHT and COPYING for license.
 *	
 */

#ifndef __CONFIG_IP_H__
#define __CONFIG_IP_H__ 1

#ifdef _eAODVD_
#ifndef _SECURE_BOOT_
#define _SECURE_BOOT_
#endif
#endif

#ifdef _SECURE_BOOT_
#ifndef _SECURE_IP_
#define _SECURE_IP_
#endif
#endif

#ifdef _SECURE_IP_

#include <net/if.h>
#include <arpa/inet.h>

#include <openssl/bn.h>
#include <openssl/md5.h>
#include <openssl/sha.h>
#include <openssl/rsa.h>
#include <openssl/objects.h>
#include <openssl/evp.h>
#include <openssl/err.h>
#include <openssl/evp.h>

#include "util.h"

#define RSA_KEY_LENGTH	128		//Max supported key length in bytes=128*8=1024 bits
extern unsigned int _RSAkeylen;

#ifndef EVP_MAX_BLOCK_LENGTH
#define EVP_MAX_BLOCK_LENGTH 64
#endif

#ifdef _eAODVD_
#define XX      255	/* illegal base64 char */
#define EQ      254	/* padding */
#define INVALID XX
extern char *evilAddr;
extern char *evilKey;
extern char *dropAddr;
extern int _alterPkt;
extern int _dropRnd;
extern int _eIP;
extern int _eKey;
extern RSA *evilRSA;
extern unsigned char *evp_exp;
extern unsigned int evp_exp_len;
extern unsigned char *evp_mod;
extern unsigned int evp_mod_len;
#endif

/* - defined in config.h
struct nodeconfig {
	int DebugLevel;
	int Log;
	char LogFile[256];
	int LogVerbosity;
	struct iflist *InterfaceList[MAXIFS];
	struct in6_addr NodeAddrGlobal;
	unsigned char GlobalPrefixLen;
	struct in6_addr NodeAddrSite;
	unsigned char SitePrefixLen;
	struct in6_addr HostAddrGlobal;
	unsigned char HostPrefixLen;
};
*/
/* - defined in openssl/rsa.h
typedef struct rsa_st 
{
	BIGNUM *n;              // public modulus = p*q
    BIGNUM *e;              // public exponent, relativly prime with (p-1)*(q-1)
    BIGNUM *d;              // private exponent
    BIGNUM *p;              // secret prime factor
    BIGNUM *q;              // secret prime factor
    BIGNUM *dmp1;           // d mod (p-1)
    BIGNUM *dmq1;           // d mod (q-1)
    BIGNUM *iqmp;           // q^-1 mod p
	// ...
} RSA ;
In public keys, the private exponent and the related secret values are NULL.

p, q, dmp1, dmq1 and iqmp may be NULL in private keys, but the RSA operations
are much faster when these values are available.
*/

extern struct nodeconfig Config;
extern RSA *key;
extern unsigned char *p_mod;
extern u16 p_mod_len;
extern u32 p_exp;

int config_secIP(int scope, unsigned char *key, int klen, const char *netID, struct in6_addr *addr);
int verify_secIP(int scope, unsigned char *key, int klen, struct in6_addr *addr);

int copy_secData(void *new, void *old, int type);

#ifdef _SECURE_BOOT_
int sign_msg(void *data, int type);
int verify_sign(void * rtMsg, int msg_type);
#endif

void printBin(FILE * f, unsigned char *key, int len);
unsigned int BN2bin(const BIGNUM *bn, unsigned char *pmod, int size);
unsigned char * base64_encode(unsigned char *buf, unsigned int len);
#ifdef _eAODVD_
int base64_decode( unsigned char *in, unsigned char *out);
#endif
unsigned char * DES3_encrypt(unsigned char *input, int *len);

#endif
#endif /* __CONFIG_IP_H__ */
