/*
 *	SecAODV IPv6 Project
 *
 *	See NIST-COPYRIGHT and COPYING for license.
 */

#ifndef __MESSAGES_H__
#define __MESSAGES_H__ 1

#include <netinet/in.h>
#include "config_ip.h"
#include "aodv6.h"
#include "util.h"

extern int xdbg;
extern int xtr;

/*
 * Message flags
 */
#define F_JOIN		1
#define F_REPAIR	2
#define F_GRATUITOUS	4
#define F_ACK		8
#define F_NODELETE	16
#define F_DEST_ONLY 32
#define F_UNKNOWN_SEQ 64

/*
 * Message creation functions
 */
struct aodv6_rreq *create_rreq(u32 flags, u8 hopcount, u32 bcast_id,
			       struct in6_addr *dst, u32 dst_seq, 
			       struct in6_addr *src, u32 src_seq
#ifdef _SECURE_BOOT_
				   , u8 signIt
#endif
				   );

struct aodv6_rrep *create_rrep(u32 flags, u8 prefix_len, u8 hopcount,
			       struct in6_addr *dst, u32 dst_seq,
			       struct in6_addr *src, u32 lifetime
#ifdef _SECURE_BOOT_
				   , int signIt
#endif
				   );

struct aodv6_rerr *create_rerr(u32 flags, u8 destcount, 
			       struct dest *unreach_dest);

struct aodv6_rrep_ack *create_rrep_ack(void);

/* 
 * Extension appending function for RREQ and RREP
 */
int add_ext(void **msg, size_t len, u8 type, u8 length, u8 *value);

/*
 * Message content dumps
 */
#define DBG_DUMP(f,x,t) dump_mux(f,x,t)
void dump_mux(FILE *out, void *arg, int type);

#endif /* __MESSAGES_H__ */
