/*
 *	SecAODV IPv6 Project
 *
 *	See NIST-COPYRIGHT and COPYING for license.
 */

#ifndef __RADIX_H__
#define __RADIX_H__ 1

typedef int (*walker_func_t)(void *data, void *udata);

typedef struct node {
	void *data;
	struct node *parent;
	struct node *left;
	struct node *right;
} node_t;

/*
 *  Debugging flags (statements and trace)
 */
extern int xdbg, xtr; 

/*
 * Create radix tree root
 */

node_t *create_tree(void);

/*
 * Add node to radix tree with key
 */
int node_add(node_t *tree, void *key, int keylen, void *data, int datalen);

/*
 * Lookup radix tree entry with key
 */
void *node_lookup(node_t *tree, void *key, int keylen, int wildcard);

/*
 * Walk through a tree and apply a function on data
 */
int tree_walker(node_t *tree, walker_func_t func, void *data);

/*
 * Modify radix tree entry with key
 */
int node_modify(node_t *tree, void *key, int keylen, void *data, int datalen);

/*
 * Delete radix tree entry with key
 */
void *node_delete(node_t *tree, void *key, int keylen);

#endif /* __RADIX_H__ */
