/*
 *	SecAODV IPv6 Project
 *
 *	See NIST-COPYRIGHT and COPYING for license.
 */

#ifndef __UTIL_H__
#define __UTIL_H__ 1

#include <string.h>
#include <netinet/in.h>
#include <stdio.h>

#define SCOPE_GLOBAL	0
#define SCOPE_SITE	200
#define SCOPE_LINK	253
#define SCOPE_HOST	254
#define SCOPE_NOWHERE	255

/*
 * Logging types and levels
 */
#define L_CRIT		0x0
#define L_ERR		0x1
#define L_WARN		0x2
#define L_INFO		0x4
#define L_PACKET	0x8

#define L_SEND		0x10
#define L_RECV		0x20
#define L_MEM		0x40
#define L_ROUTE		0x80
#define L_RTADD		0x100
#define L_RTMOD		0x200
#define L_RTDEL		0x400
#define L_RTSYNC	0x800

#define L_SEND_INFO	L_SEND | L_INFO
#define L_SEND_PACKET	L_SEND | L_PACKET
#define L_RECV_INFO	L_RECV | L_INFO
#define L_RECV_PACKET	L_RECV | L_PACKET
#define L_RTMOD_INFO	L_RTMOD | L_INFO

#define L_EVERYTHING	0xffff

/*
#ifdef DEBUG_AODV
#define DEBUG_FN fprintf(stdout, "AODVD: === " __FUNCTION__ " ===\n")
#else
#define DEBUG_FN
#endif
*/


/*
 * Few life easing typedefs.
 */
typedef u_int8_t  u8;
typedef u_int16_t u16;
typedef u_int32_t u32;

/*
 * Debug levels to be used with dprint()
 */
enum {
	DBGCRIT = 0,
	DBGERROR = 1,
	DBGWARN = 2,
	DBGINFO = 3
};

/*
 * Print struct in6_addr.
 */
void print_in6addr(FILE *f, const struct in6_addr *addr);
void print_ntoh_in6addr(FILE *f, struct in6_addr *addr);

#define NIP6ADDR(addr) \
        ntohs(((u16 *)addr)[0]), \
        ntohs(((u16 *)addr)[1]), \
        ntohs(((u16 *)addr)[2]), \
        ntohs(((u16 *)addr)[3]), \
        ntohs(((u16 *)addr)[4]), \
        ntohs(((u16 *)addr)[5]), \
        ntohs(((u16 *)addr)[6]), \
        ntohs(((u16 *)addr)[7])

/*
 * Calculate hash key
 */
u_int32_t in6_hash(struct in6_addr *addr);

/*
 * Convert in6_addr from network byte order to host byte order
 */
struct in6_addr *ntoh_in6_addr(struct in6_addr *addr, struct in6_addr *buf);

/*
 * Convert in6_addr from host byte order to network byte order
 */
struct in6_addr *hton_in6_addr(struct in6_addr *addr, struct in6_addr *buf);

static inline int ipv6_addr_cmp(struct in6_addr *a1, struct in6_addr *a2)
{
	return memcmp((void *) a1, (void *) a2, sizeof(struct in6_addr));
}

static inline void ipv6_addr_copy(struct in6_addr *a1, struct in6_addr *a2)
{
	memcpy((void *) a1, (void *) a2, sizeof(struct in6_addr));
}

unsigned long timems(void);

void dlog(int level, const char* fmt, ...);

int ulogif(int level);
void ulog(int level, const char *fmt, ...);

#define ulog_if(x) \
	if (ulogif(x))

#define max(a,b) a >= b ? a : b
#define min(a,b) a <= b ? a : b

#endif /* __UTIL_H__ */
